'#####################################################################################################################
'**************************************************Class: Common XML-Handle ******************************************
'********************AUTHOR: Erich Lieberum, UMGIS Informatik GmbH, DArmstadt, Germany 2005***************************
'********************In friendly abandonment for non commercial use***************************************************
'********************The class is ready for read/write use************************************************************
'ABSTRACT:
'The program so called "ArcGIS-map to SLD Converter" analyses an
'ArcMap-Project with respect to its symbolisation and assembles an SLD
'for the OGC-Web Map Service (WMS) from the gathered data. The program
'is started parallel to a running ArcMap 9.X-session. Subsequently the
'application deposits an SLD-file which complies the symbolisation of
'the available ArcMap-project. With the SLD a WMS-project may be
'classified and styled according to the preceding ArcMap-project. The
'application is written in VisualBasic.NET and uses the .NET 1.1
'Framework (plus XML files for configuration). For more informtion
'refer to: 
'http://arcmap2sld.geoinform.fh-mainz.de/ArcMap2SLDConverter_Eng.htm.
'LICENSE:
'This program is free software under the license of the GNU Lesser General Public License (LGPL) As published by the Free Software Foundation. 
'With the use and further development of this code you accept the terms of LGPL. For questions of the License refer to:
'http://www.gnu.org/licenses/lgpl.html
'DISCLAIMER:
'THE USE OF THE SOFTWARE ArcGIS-map to SLD Converter HAPPENS AT OWN RISK.
'I CANNOT ISSUE A GUARANTEE FOR ANY DISADVANTAGES (INCLUDING LOSS OF DATA; ETC.) THAT
'MAY ARISE FROM USING THIS SOFTWARE.
'DESCRIPTION:
'The class provides XML-functions for a simple XML-doc with max. 3 hierarchy levels. The levels are called: 1.Section
'2. Entry, 3.Subentry. The Document Element is called "Root"
'CHANGES:
'#####################################################################################################################

Imports System.Xml
Imports System.IO
Public Class CommonXMLHandle
    Private cXMLFileName As String
    Private objDOC As XmlDocument
    Private bDocIsOpen As Boolean
    Private objActiveRoot As XmlElement
    Private objActiveSection As XmlElement
    Private objActiveEntry As XmlElement
    Private objActiveSubEntry As XmlElement
    Private objActiveAttribute As XmlAttribute
    Private objActiveSubentryAttribute As XmlAttribute
    Private cMessage As String
    Private DocMode As XmlMode

    Private Enum XmlMode
        xmlDocClosed = 0
        xmlDocOpen = 1
        xmlSectionSelected = 2
        xmlEntrySelected = 3
        xmlRootSelected = 4
    End Enum

    Public Sub New()
        bDocIsOpen = False
        DocMode = XmlMode.xmlDocClosed
    End Sub

    ' Name des XML-Dokuments
    Public Property XMLfilename() As String
        Get
            XMLfilename = cXMLFileName
        End Get
        Set(ByVal Value As String)
            cXMLFileName = Value
        End Set
    End Property

    ' Auftretende Fehlermeldungen
    Public ReadOnly Property ErrorMessage() As String
        Get
            ErrorMessage = cMessage
        End Get
    End Property

    ' ffnen des XML-Dokuments
    Public Function OpenDoc() As Boolean
        Try
            objDOC = New XmlDocument
            objDOC.Load(cXMLFileName)
            bDocIsOpen = True
            DocMode = XmlMode.xmlDocOpen
        Catch ex As Exception
            cMessage = "Fehler beim ffnen des Dokuments (" & ex.Message.ToString & ")"
            Return False
        End Try
        Return True
    End Function
    ' Speichern des XML-Dokuments
    Public Function SaveDoc() As Boolean
        Try
            objDOC.Save(cXMLFileName)
        Catch ex As Exception
            cMessage = "Fehler beim Speichern des Dokuments (" & ex.Message.ToString & ")"
            Return False
        End Try
        Return True
    End Function

    '********************************************************************************************
    ' Suchen einer Root-Knoten (0. Ebene)
    ' -------------------------------
    ' Rckgabewerte: True = Root-Knoten  wurde gefunden
    '                false = Root-Knoten  wurde nicht gefunden
    ' Zeiger sitzt auf gefundenem Root-Knoten 
    '********************************************************************************************
    Public Function FindRoot(ByVal RootName As String) As Boolean
        If bDocIsOpen = False Then
            cMessage = ("Dokument ist nicht geffnet!")
            Return False
        End If
        Dim i As Integer = 0

        Dim objNodeRoot As XmlNode
        objNodeRoot = objDOC.FirstChild
        If TypeOf objNodeRoot Is XmlDeclaration Then
            objNodeRoot = objDOC.SelectSingleNode(RootName)
        End If

        Do Until objNodeRoot Is Nothing
            If objNodeRoot.Name.ToUpper = RootName.ToUpper Then
                objActiveRoot = objNodeRoot
                DocMode = XmlMode.xmlRootSelected
                cMessage = ""
                Return True
            End If
            objNodeRoot = objDOC.NextSibling
        Loop
        cMessage = ("Root-Knoten nicht vorhanden!")
        Return False

    End Function
    '********************************************************************************************
    ' Suchen einer Sektion (1. Ebene)
    ' -------------------------------
    ' Rckgabewerte: True = Sektion wurde gefunden
    '                false = Sektion wurde nicht gefunden
    ' Zeiger sitzt auf gefundenem Knoten
    '********************************************************************************************
    Public Function FindSection(ByVal Section As String) As Boolean
        If bDocIsOpen = False Then
            cMessage = ("Dokument ist nicht geffnet!")
            Return False
        End If
        Dim i As Integer = 0
        'Dim objRoot As XmlNode
        'objRoot = objDOC.FirstChild

        Dim objNodeSection As XmlNode
        If objActiveRoot Is Nothing Then
            objActiveRoot = objDOC.FirstChild
        End If
        objNodeSection = objActiveRoot.FirstChild
        Do Until objNodeSection Is Nothing
            If objNodeSection.Name.ToUpper = Section.ToUpper Then
                objActiveSection = objNodeSection
                DocMode = XmlMode.xmlSectionSelected
                cMessage = ""
                Return True
            End If
            objNodeSection = objNodeSection.NextSibling
        Loop
        cMessage = ("Sektion nicht vorhanden!")
        Return False

    End Function


    '********************************************************************************************
    ' Suchen eines Sektions-Attributs (1. Ebene)
    ' -------------------------------
    ' Rckgabewerte: True = Attribut wurde gefunden
    '                false = Attribut wurde nicht gefunden
    ' Zeiger sitzt auf gefundenem Attribut
    ' Eine Sektion muss vorher ausgewhlt werden
    '********************************************************************************************
    Public Function FindSectionAttribute(ByVal SectionAttributeName As String) As Boolean
        If Not DocMode = XmlMode.xmlSectionSelected Then
            cMessage = ("Keine Eintrag ausgewhlt!")
            Return False
        End If
        Dim i As Integer = 0

        Dim objSectionAttribute As XmlAttribute
        If objActiveSection.Attributes.Count = 0 Then
            cMessage = ("Keine Attribute vorhanden!")
            Return False
        End If
        objSectionAttribute = objActiveEntry.Attributes(0)
        For Each objSectionAttribute In objActiveSection.Attributes
            If objSectionAttribute.Name.ToUpper = SectionAttributeName.ToUpper Then
                objActiveAttribute = objSectionAttribute
                DocMode = XmlMode.xmlSectionSelected
                cMessage = ""
                Return True
            End If
        Next
        cMessage = ("Attribut nicht vorhanden!")
        Return False

    End Function

    '********************************************************************************************
    ' berprfung, ob berhaupt irgendeine Sektion angelegt wurde
    ' -------------------------------
    ' Rckgabewerte: True = eine Sektion wurde gefunden
    '                false = keune Sektion wurde gefunden
    ' Zeiger sitzt auf gefundenem Knoten
    '********************************************************************************************

    '********************************************************************************************
    Public Function FindAnySection() As Boolean
        Try
            If bDocIsOpen = False Then
                cMessage = ("Dokument ist nicht geffnet!")
                Return False
            End If
            If objActiveRoot Is Nothing Then
                objActiveRoot = objDOC.FirstChild
            End If
            If objActiveRoot.FirstChild Is Nothing Then
                Return False
            Else
                objActiveSection = objActiveRoot.FirstChild
                Return True
            End If

        Catch ex As Exception
            cMessage = ("Fehler in Funktion FindAnySection")
        End Try
    End Function



    ' Suchen eines Eintrags (2. Ebene)
    ' -------------------------------
    ' Rckgabewerte: True = Eintrag wurde gefunden
    '                false = Eintrag wurde nicht gefunden
    ' Zeiger sitzt auf gefundenem Knoten
    ' Eine Sektion muss vorher ausgewhlt werden
    '********************************************************************************************
    Public Function FindEntry(ByVal Entry As String) As Boolean
        If DocMode = XmlMode.xmlDocClosed Or DocMode = XmlMode.xmlDocOpen Then
            cMessage = ("Keine Sektion ausgewhlt!")
            Return False
        End If
        Dim i As Integer = 0

        Dim objNodeEntry As XmlNode

        objNodeEntry = objActiveSection.FirstChild
        Do Until objNodeEntry Is Nothing
            If objNodeEntry.Name.ToUpper = Entry.ToUpper Then
                objActiveEntry = objNodeEntry
                DocMode = XmlMode.xmlEntrySelected
                cMessage = ""
                Return True
            End If
            objNodeEntry = objNodeEntry.NextSibling
        Loop
        cMessage = ("Eintrag nicht vorhanden!")
        Return False

    End Function

    '********************************************************************************************
    ' Suchen eines Attributs (2. Ebene)
    ' -------------------------------
    ' Rckgabewerte: True = Attribut wurde gefunden
    '                false = Attribut wurde nicht gefunden
    ' Zeiger sitzt auf gefundenem Attribut
    ' Ein Eintrag muss vorher ausgewhlt werden
    '********************************************************************************************
    Public Function FindAttribute(ByVal AttributeName As String) As Boolean
        If Not DocMode = XmlMode.xmlEntrySelected Then
            cMessage = ("Keine Eintrag ausgewhlt!")
            Return False
        End If
        Dim i As Integer = 0

        Dim objAttribute As XmlAttribute
        If objActiveEntry.Attributes.Count = 0 Then
            cMessage = ("Keine Attribute vorhanden!")
            Return False
        End If
        objAttribute = objActiveEntry.Attributes(0)
        For Each objAttribute In objActiveEntry.Attributes
            If objAttribute.Name.ToUpper = AttributeName.ToUpper Then
                objActiveAttribute = objAttribute
                DocMode = XmlMode.xmlEntrySelected
                cMessage = ""
                Return True
            End If
        Next
        cMessage = ("Attribut nicht vorhanden!")
        Return False

    End Function


    '********************************************************************************************
    ' Abrufen eines Eintrags-Wertes
    ' -----------------------------
    '********************************************************************************************
    Public Function getEntryValue() As String
        Try
            cMessage = ""
            Return objActiveEntry.InnerText
        Catch ex As Exception
            cMessage = "Kein Wert gefunden (" & ex.Message.ToString & ")"
            Return ""
        End Try
    End Function
    'berladen:
    Public Function getEntryValue(ByVal Entryname As String) As String
        Try
            cMessage = ""
            FindEntry(Entryname)
            Return objActiveEntry.InnerText
        Catch ex As Exception
            cMessage = "Kein Wert gefunden (" & ex.Message.ToString & ")"
            Return ""
        End Try
    End Function

    '********************************************************************************************
    ' Abrufen eines Eintragsnamens
    ' ---------------------------------------
    '********************************************************************************************
    Public Function getEntryName() As String
        Try
            cMessage = ""
            Return objActiveEntry.Name
        Catch ex As Exception
            cMessage = "Kein Name gefunden (" & ex.Message.ToString & ")"
            Return ""
        End Try
    End Function

    '********************************************************************************************
    ' Abrufen eines Section-Namens
    ' -----------------------------
    '********************************************************************************************
    Public Function getSectionName() As String
        Try
            cMessage = ""
            Return objActiveSection.Name
        Catch ex As Exception
            cMessage = "Kein Name gefunden (" & ex.Message.ToString & ")"
            Return ""
        End Try
    End Function

    '********************************************************************************************
    ' Abrufen eines Sektionswertes
    ' aktive Sektion
    ' ---------------------------------------
    '********************************************************************************************
    Public Function getSectionValue() As String
        Try
            cMessage = ""
            Return objActiveSection.InnerText
        Catch ex As Exception
            cMessage = "Kein Name gefunden (" & ex.Message.ToString & ")"
            Return ""
        End Try
    End Function

    '********************************************************************************************
    ' Abrufen eines Attributwertes
    ' aktives Attribut
    ' ---------------------------------------
    '********************************************************************************************
    Public Function getAttributeValue() As String
        Try
            cMessage = ""
            Return objActiveAttribute.Value
        Catch ex As Exception
            cMessage = "Kein Wert gefunden (" & ex.Message.ToString & ")"
            Return ""
        End Try
    End Function
    'berladen:
    Public Function getAttributeValue(ByVal AttributName As String) As String
        Try
            cMessage = ""
            FindAttribute(AttributName)
            Return objActiveAttribute.Value
        Catch ex As Exception
            cMessage = "Kein Wert gefunden (" & ex.Message.ToString & ")"
            Return ""
        End Try
    End Function


    '********************************************************************************************
    ' Abrufen eines Attributnamens
    ' aktives Attribut
    ' ---------------------------------------
    '********************************************************************************************
    Public Function getAttributeName() As String
        Try
            cMessage = ""
            Return objActiveAttribute.Name
        Catch ex As Exception
            cMessage = "Kein Name gefunden (" & ex.Message.ToString & ")"
            Return ""
        End Try
    End Function

    '********************************************************************************************
    ' Setzen eines Sektionswertes
    ' aktiver Sektion
    ' ---------------------------------------
    '********************************************************************************************
    Public Function SetSectionValue(ByVal NewValue As String) As Boolean
        Try
            cMessage = ""
            objActiveSection.InnerText = NewValue
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Schreiben (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function


    '********************************************************************************************
    ' Setzen eines Eintragswertes
    ' aktiver Eintrag
    ' ---------------------------------------
    '********************************************************************************************
    Public Function SetEntryValue(ByVal NewValue As String) As Boolean
        Try
            cMessage = ""
            objActiveEntry.InnerText = NewValue
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Schreiben (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function

    '********************************************************************************************
    ' Setzen eines Attributwertes
    ' aktives Attribut
    ' ---------------------------------------
    '********************************************************************************************
    Public Function SetAttributeValue(ByVal NewValue As String) As Boolean
        Try
            cMessage = ""
            objActiveAttribute.Value = NewValue
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Schreiben (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function

    '********************************************************************************************
    ' Lschen einer Sektion
    ' aktiver Eintrag
    ' ---------------------------------------
    '********************************************************************************************
    Public Function DeleteSection() As Boolean
        Try
            objActiveRoot.RemoveChild(objActiveSection)
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Lschen des Entrys (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function

    '********************************************************************************************
    ' Lschen eines Eintrags
    ' aktiver Eintrag
    ' ---------------------------------------
    '********************************************************************************************
    Public Function DeleteEntry() As Boolean
        Try
            objActiveSection.RemoveChild(objActiveEntry)
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Lschen des Eintrags (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function
    '********************************************************************************************
    ' Lschen eines Attributs
    ' aktives Attribut
    ' ---------------------------------------
    '********************************************************************************************
    Public Function DeleteAttribute() As Boolean
        Try
            objActiveEntry.Attributes.Remove(objActiveAttribute)
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Lschen des Attributs (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function

    '********************************************************************************************
    ' Anlegen einer Sektion
    ' die Neue Sektion entspricht der aktiven Sektion
    ' ---------------------------------------
    '********************************************************************************************
    Public Function CreateSection(ByVal Section As String) As Boolean
        Try
            'Dim objRoot As XmlElement = objDOC.FirstChild
            If objActiveRoot Is Nothing Then
                objActiveRoot = objDOC.FirstChild
            End If
            Dim objNewSection As XmlElement
            objNewSection = objDOC.CreateElement(Section)
            objActiveRoot.AppendChild(objNewSection)
            objActiveSection = objNewSection
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Anlegen einer neuen Sektion (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function


    '********************************************************************************************
    ' Anlegen eines Attributs zur Sektion
    ' neues Attribut = aktives Attribut
    ' Sektion muss vorher gewhlt sein
    '********************************************************************************************
    Public Function CreateSectionAttribute(ByVal AttributeName As String) As Boolean
        Try
            Dim objNewSectionAttribute As XmlAttribute
            objNewSectionAttribute = objDOC.CreateAttribute(AttributeName)
            objActiveSection.Attributes.Append(objNewSectionAttribute)
            objActiveAttribute = objNewSectionAttribute
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Anlegen eines neuen Attributs (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function


    '********************************************************************************************
    ' Anlegen eines Eintrags
    ' der neue Eintrag = der aktiver Eintrag
    ' Sektion muss vorher gewhlt sein
    '********************************************************************************************
    Public Function CreateEntry(ByVal Entry As String) As Boolean
        Try
            Dim objNewEntry As XmlElement
            objNewEntry = objDOC.CreateElement(Entry)
            objActiveSection.AppendChild(objNewEntry)
            objActiveEntry = objNewEntry
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Anlegen einer neuen Sektion (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function
    '********************************************************************************************
    ' Anlegen eines Attributs
    ' neues Attribut = aktives Attribut
    ' Sektion und Eintrag mssen vorher gewhlt sein
    '********************************************************************************************
    Public Function CreateAttribute(ByVal AttributeName As String) As Boolean
        Try
            Dim objNewAttribute As XmlAttribute
            objNewAttribute = objDOC.CreateAttribute(AttributeName)
            objActiveEntry.Attributes.Append(objNewAttribute)
            objActiveAttribute = objNewAttribute
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Anlegen eines neuen Attributs (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function
    Protected Overrides Sub Finalize()
        MyBase.Finalize()
    End Sub
    '********************************************************************************************
    ' Anzahl der Eintrge in einer Sektion
    '********************************************************************************************
    Public ReadOnly Property EntryCount() As Integer
        Get
            If objActiveEntry Is Nothing Then
                Return -1
            Else
                Return objActiveEntry.ChildNodes.Count
            End If
        End Get
    End Property

    '********************************************************************************************
    ' Erstellen eines Arrays mit allen Eintragsnamen in einer Sektion
    ' ---------------------------------------------------------------
    ' Rckgabewerte: Array = Alle Eintragsnamen
    '                Nothing = Fehler
    ' Zeiger sitzt auf letztem Eintrag
    ' Eine Sektion muss vorher ausgewhlt werden
    '********************************************************************************************
    Public Function getEntryArray() As Array
        Try

            Dim nNodeCount As Integer = objActiveSection.ChildNodes.Count
            Dim tmpArray As Array = tmpArray.CreateInstance(GetType(String), nNodeCount)

            Dim objNodeEntry As XmlNode

            objNodeEntry = objActiveSection.FirstChild
            Dim i As Integer = 0
            Do Until objNodeEntry Is Nothing
                objActiveEntry = objNodeEntry
                tmpArray.SetValue(objNodeEntry.Name, i)
                i += 1
                objNodeEntry = objNodeEntry.NextSibling
            Loop
            Return (tmpArray)

        Catch ex As Exception
            cMessage = "Fehler beim Erstellen des Arrays (" & ex.Message.ToString & ")"
            Return Nothing
        End Try

    End Function
    'berladen
    Public Function getEntryArray(ByVal EntryName As String) As Array
        Try
            FindEntry(EntryName)
            Dim nNodeCount As Integer = objActiveSection.ChildNodes.Count
            Dim tmpArray As Array = tmpArray.CreateInstance(GetType(String), nNodeCount)

            Dim objNodeEntry As XmlNode

            objNodeEntry = objActiveSection.FirstChild
            Dim i As Integer = 0
            Do Until objNodeEntry Is Nothing
                objActiveEntry = objNodeEntry
                tmpArray.SetValue(objNodeEntry.Name, i)
                i += 1
                objNodeEntry = objNodeEntry.NextSibling
            Loop
            Return (tmpArray)

        Catch ex As Exception
            cMessage = "Fehler beim Erstellen des Arrays (" & ex.Message.ToString & ")"
            Return Nothing
        End Try

    End Function

    '********************************************************************************************
    ' Erstellen eines Arrays mit allen InnerText-Elementen der Entrys in einer Sektion
    ' ---------------------------------------------------------------
    ' Rckgabewerte: Array = Alle InnerText-Elemente
    '                Nothing = Fehler
    ' Zeiger sitzt auf letztem Eintrag
    ' Eine Sektion muss vorher ausgewhlt werden
    '********************************************************************************************
    Public Function getInnerTextArray() As Array
        Try

            Dim nNodeCount As Integer = objActiveSection.ChildNodes.Count
            Dim tmpArray As Array = tmpArray.CreateInstance(GetType(String), nNodeCount)

            Dim objNodeEntry As XmlNode

            objNodeEntry = objActiveSection.FirstChild
            Dim i As Integer = 0
            Do Until objNodeEntry Is Nothing
                objActiveEntry = objNodeEntry
                tmpArray.SetValue(objNodeEntry.InnerText, i)
                i += 1
                objNodeEntry = objNodeEntry.NextSibling
            Loop
            Return (tmpArray)

        Catch ex As Exception
            cMessage = "Fehler beim Erstellen des Arrays (" & ex.Message.ToString & ")"
            Return Nothing
        End Try

    End Function


    '********************************************************************************************
    ' Erstellen eines Arrays mit allen Sections einer XML-Datei
    ' ---------------------------------------------------------------
    ' Rckgabewerte: Array = Alle Eintragsnamen
    '                Nothing = Fehler
    ' Zeiger sitzt auf letztem Eintrag
    ' Eine Sektion muss vorher ausgewhlt werden
    '********************************************************************************************
    Public Function GetSectionArray() As Array
        Try
            Dim objNodeRoot As XmlNode
            objNodeRoot = objDOC.FirstChild
            If objActiveRoot Is Nothing Then
                objActiveRoot = objNodeRoot
            End If
            Dim nNodeCount As Integer = objNodeRoot.ChildNodes.Count
            Dim tmpArray As Array = tmpArray.CreateInstance(GetType(String), nNodeCount)
            Dim objNodeSection As XmlElement


            objNodeSection = objNodeRoot.FirstChild
            Dim i As Integer = 0
            Do Until objNodeSection Is Nothing
                objActiveSection = objNodeSection
                tmpArray.SetValue(objNodeSection.Name, i)
                i += 1
                objNodeSection = objNodeSection.NextSibling
            Loop
            Return (tmpArray)
        Catch ex As Exception
            cMessage = "Fehler beim Erstellen des Arrays (" & ex.Message.ToString & ")"
        End Try
    End Function


    '********************************************************************************************
    ' Erstellen eines Arrays mit allen Attributnamen eines Entrys
    ' ---------------------------------------------------------------
    ' Rckgabewerte: Array = Alle Eintragsnamen
    '                Nothing = Fehler
    ' Zeiger sitzt auf letztem Eintrag
    ' Eine Sektion muss vorher ausgewhlt werden
    '********************************************************************************************
    Public Function getAttributeNameArray() As Array
        Try

            Dim nAttributeCount As Integer = objActiveEntry.Attributes.Count
            Dim tmpArray As Array = tmpArray.CreateInstance(GetType(String), nAttributeCount)

            Dim objEntryAttribute As XmlAttribute

            If nAttributeCount > 0 Then
                Dim i As Integer = 0
                For Each objEntryAttribute In objActiveEntry.Attributes
                    tmpArray.SetValue(objEntryAttribute.Name, i)
                    i += 1
                Next
            End If
            Return (tmpArray)

        Catch ex As Exception
            cMessage = "Fehler beim Erstellen des Arrays (" & ex.Message.ToString & ")"
            Return Nothing
        End Try

    End Function


    '********************************************************************************************
    ' Erstellen eines Arrays mit allen Attributwerten eines Entrys
    ' ---------------------------------------------------------------
    ' Rckgabewerte: Array = Alle Eintragsnamen
    '                Nothing = Fehler
    ' Zeiger sitzt auf letztem Eintrag
    ' Eine Sektion muss vorher ausgewhlt werden
    '********************************************************************************************
    Public Function getAttributeArray() As Array
        Try

            Dim nAttributeCount As Integer = objActiveEntry.Attributes.Count
            Dim tmpArray As Array = tmpArray.CreateInstance(GetType(String), nAttributeCount)

            Dim objEntryAttribute As XmlAttribute

            If nAttributeCount > 0 Then
                Dim i As Integer = 0
                For Each objEntryAttribute In objActiveEntry.Attributes
                    tmpArray.SetValue(objEntryAttribute.Value, i)
                    i += 1
                Next
            End If
            Return (tmpArray)

        Catch ex As Exception
            cMessage = "Fehler beim Erstellen des Arrays (" & ex.Message.ToString & ")"
            Return Nothing
        End Try

    End Function


    '********************************************************************************************
    ' Erstellen einer neuen XML-Datei
    ' ---------------------------------------------------------------
    ' Voraussetzung : Der Dateiname muss bekannt sein (Property "XMLfilename")
    ' Parameter :
    '       RootName = Name des Stammverzeichnises
    '       OverWrite = Soll bestehende Datei berschrieben werden (Ja / Nein)
    ' Rckgabewerte: 
    '       true = Ausfhrung korrekt; Datei wurde erstellt
    '       false = Bei der Ausfhrung trat ein Fehler auf
    ' Zeiger sitzt auf Root-Knoten
    '********************************************************************************************
    Public Function CreateNewFile(ByVal RootName As String, ByVal OverWrite As Boolean) As Boolean
        Try
            If RootName = "" Or RootName Is Nothing Then
                cMessage = "Fehler beim Anlegen der XML-Datei (Knotenname muss angegeben werden)"
                Return False

            End If
            If File.Exists(cXMLFileName) = True Then
                ' cXMLfilenameTMP = AppDomain.CurrentDomain.BaseDirectory & "\UMGIS_ADR_Control.CFG"
                If OverWrite = False Then
                    cMessage = "Fehler beim Anlegen der XML-Datei (Datei besteht bereits)"
                    Return False
                End If
                File.Delete(cXMLFileName)
            End If

            ' If File.Exists(cXMLfilenameTMP) = False Then
            ' XML-datei anlegen
            Dim objXMLdoc As New XmlDocument
            Dim objRoot As XmlElement

            objRoot = objXMLdoc.CreateElement(RootName)

            objXMLdoc.AppendChild(objRoot)

            objActiveRoot = objRoot

            objXMLdoc.Save(cXMLFileName)

            objDOC = objXMLdoc
            bDocIsOpen = True
            DocMode = XmlMode.xmlDocOpen

            Return True

        Catch ex As Exception
            cMessage = "Fehler beim Anlegen der XML-Datei (" & ex.Message.ToString & ")"
            Return Nothing
        End Try
    End Function
    '********************************************************************************************
    ' Erstellen einer neuen XML-Datei
    ' ---------------------------------------------------------------
    ' Voraussetzung : Der Dateiname muss bekannt sein (Property "XMLfilename")
    ' Parameter :
    '       RootName = Name des Stammverzeichnises
    '       OverWrite = Soll bestehende Datei berschrieben werden (Ja / Nein)
    '       CodeNode = Code
    ' Rckgabewerte: 
    '       true = Ausfhrung korrekt; Datei wurde erstellt
    '       false = Bei der Ausfhrung trat ein Fehler auf
    ' Zeiger sitzt auf Root-Knoten
    '********************************************************************************************
    Public Function CreateNewFile(ByVal RootName As String, ByVal OverWrite As Boolean, _
            ByVal XMLVersion As String, ByVal XMLEncoding As String, _
            ByVal XMLStandAlone As String) As Boolean
        Try
            If RootName = "" Or RootName Is Nothing Then
                cMessage = "Fehler beim Anlegen der XML-Datei (Knotenname muss angegeben werden)"
                Return False

            End If
            If File.Exists(cXMLFileName) = True Then
                ' cXMLfilenameTMP = AppDomain.CurrentDomain.BaseDirectory & "\UMGIS_ADR_Control.CFG"
                If OverWrite = False Then
                    cMessage = "Fehler beim Anlegen der XML-Datei (Datei besteht bereits)"
                    Return False
                End If
                File.Delete(cXMLFileName)
            End If

            ' If File.Exists(cXMLfilenameTMP) = False Then
            ' XML-datei anlegen
            Dim objXMLdoc As New XmlDocument
            Dim objRoot As XmlElement
            Dim objXMLdecl As XmlDeclaration

            'Add the new node to the document.

            objXMLdecl = objXMLdoc.CreateXmlDeclaration(XMLVersion, XMLEncoding, XMLStandAlone)

            'objXMLdoc.CreateXmlDeclaration(XMLVersion, XMLEncoding, XMLStandAlone)

            'objRoot = objXMLdoc.CreateElement(CodeNodeName)
            'objXMLdoc.AppendChild(objRoot)

            objRoot = objXMLdoc.CreateElement(RootName)
            objXMLdoc.AppendChild(objRoot)

            objActiveRoot = objRoot

            objXMLdoc.InsertBefore(objXMLdecl, objRoot)

            objXMLdoc.Save(cXMLFileName)

            objDOC = objXMLdoc
            bDocIsOpen = True
            DocMode = XmlMode.xmlDocOpen

            Return True

        Catch ex As Exception
            cMessage = "Fehler beim Anlegen der XML-Datei (" & ex.Message.ToString & ")"
            Return Nothing
        End Try

    End Function


    '********************************************************************************************
    'Funktion legt die zu dem bergebenen AttributValue zugehrenden Active Section, Active Entry
    'und Active Attribute fest
    ' ---------------------------------------------------------------
    'Voraussetzung:XMLOpen und DateiPfadString wurde bergeben
    ' Rckgabewerte: Bool
    '                FALSE = Fehler
    ' Zeiger sitzt auf letztem Eintrag
    '********************************************************************************************
    Public Function SetSectionEntryAttributeActive(ByVal cAttributeValue As String) As Boolean
        Try
            Dim SectionArray, EntryArray, AttributeNameArray As Array
            Dim i, j, k As Integer
            Dim bSwitch As Boolean
            bSwitch = False

            SectionArray = GetSectionArray()
            For i = SectionArray.GetLowerBound(0) To SectionArray.GetUpperBound(0)
                FindSection(SectionArray(i))
                EntryArray = getEntryArray()
                For j = EntryArray.GetLowerBound(0) To EntryArray.GetUpperBound(0)
                    FindEntry(EntryArray(j))
                    AttributeNameArray = getAttributeNameArray()
                    For k = AttributeNameArray.GetLowerBound(0) To AttributeNameArray.GetUpperBound(0)
                        FindAttribute(AttributeNameArray(k))
                        If cAttributeValue = getAttributeValue() Then
                            bSwitch = True
                            Return True
                        End If
                    Next
                Next
            Next
            If bSwitch = False Then
                Return False
            End If
        Catch ex As Exception
            cMessage = "Fehler bei der Auswahl der Section, Entry, Attribute(" & ex.Message.ToString & ")"
            Return Nothing
        End Try


    End Function

    '********************************************************************************************
    ' Anzahl der Attributes in eines Eintrags
    '********************************************************************************************
    Public ReadOnly Property AttributeCount() As Integer
        Get
            If objActiveEntry Is Nothing Then
                Return -1
            Else
                Return objActiveEntry.Attributes.Count
            End If
        End Get
    End Property

    '********************************************************************************************
    ' Gibt die Inhalte aller Entries einer Sektion aus in eine Arraylist
    'Voraussetzung: eine Section wurde aktiv gesetzt
    '********************************************************************************************
    Public Function GetAllEntryValues() As ArrayList
        Dim Al As New ArrayList
        Dim oNode As XmlNode
        Try
            If Not objActiveSection Is Nothing Then
                oNode = objActiveSection.FirstChild
                Do Until oNode Is Nothing
                    Al.Add(oNode.InnerText)
                    oNode = oNode.NextSibling
                Loop
            Else
                cMessage = ("Section wurde nicht aktiv gesetzt")
                Exit Function
            End If
            Al.Sort()
            Return Al
        Catch ex As Exception
            cMessage = ("Fehler im CommonXMLHandle/GetAllEntryInnerText")
        End Try
    End Function



    '####################################################################################################
    '######################################Die 3. Dimension-Subentries###################################
    '####################################################################################################
    '********************************************************************************************
    ' Weiterentwicklung der Klasse clsXMLHandle
    ' Neu:
    ' * 3. Ebene (SubEntry)
    ' * Attribute auch fr 1. und 3. Ebene (Sektion und SubEntry)
    ' Erich Lieberum (03.02.2004)
    '********************************************************************************************


    '********************************************************************************************
    ' Suchen eines Unter-Eintrags (3. Ebene)
    ' -------------------------------
    ' Rckgabewerte: True = Unter-Eintrag wurde gefunden
    '                false = Unter-Eintrag wurde nicht gefunden
    ' Zeiger sitzt auf gefundenem Knoten
    ' Eine Eintrag muss vorher ausgewhlt werden
    '********************************************************************************************
    Public Function FindSubEntry(ByVal SubEntry As String) As Boolean
        If Not DocMode = XmlMode.xmlEntrySelected Then
            cMessage = ("Keine Eintrag ausgewhlt!")
            Return False
        End If
        Dim i As Integer = 0

        Dim objNodeSubEntry As XmlNode

        objNodeSubEntry = objActiveEntry.FirstChild
        Do Until objNodeSubEntry Is Nothing
            If objNodeSubEntry.Name.ToUpper = SubEntry.ToUpper Then
                objActiveSubEntry = objNodeSubEntry
                DocMode = XmlMode.xmlEntrySelected
                cMessage = ""
                Return True
            End If
            objNodeSubEntry = objNodeSubEntry.NextSibling
        Loop
        cMessage = ("Eintrag nicht vorhanden!")
        Return False

    End Function

    '********************************************************************************************
    ' Anlegen eines Unter-Eintrags
    ' der neue Unter-Eintrag = der aktiver Unter-Eintrag
    ' Eintrag muss vorher gewhlt sein
    '********************************************************************************************
    Public Function CreateSubEntry(ByVal SubEntry As String) As Boolean
        Try
            Dim objNewSubEntry As XmlElement
            objNewSubEntry = objDOC.CreateElement(SubEntry)
            objActiveEntry.AppendChild(objNewSubEntry)
            objActiveSubEntry = objNewSubEntry
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Anlegen einer neuen Sektion (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function


    '********************************************************************************************
    ' Anlegen eines Attributs zum Unter-Eintrag
    ' neues Attribut = aktives Attribut
    ' Unter-Eintrag muss vorher gewhlt sein
    '********************************************************************************************
    Public Function CreateSubEntryAttribute(ByVal AttributeName As String) As Boolean
        Try
            Dim objNewSubEntryAttribute As XmlAttribute
            objNewSubEntryAttribute = objDOC.CreateAttribute(AttributeName)
            objActiveSubEntry.Attributes.Append(objNewSubEntryAttribute)
            objActiveSubentryAttribute = objNewSubEntryAttribute
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Anlegen eines neuen Attributs (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function


    '********************************************************************************************
    ' Suchen eines Untereintrags-Attributs (3. Ebene)
    ' -------------------------------
    ' Rckgabewerte: True = Attribut wurde gefunden
    '                false = Attribut wurde nicht gefunden
    ' Zeiger sitzt auf gefundenem Attribut
    ' Ein Untereintrag muss vorher ausgewhlt werden
    '********************************************************************************************
    Public Function FindSubEntryAttribute(ByVal SubEntryAttributeName As String) As Boolean
        If Not DocMode = XmlMode.xmlEntrySelected Then
            cMessage = ("Keine Eintrag ausgewhlt!")
            Return False
        End If
        Dim i As Integer = 0

        Dim objSubEntryAttribute As XmlAttribute
        If objActiveSubEntry.Attributes.Count = 0 Then
            cMessage = ("Keine Attribute vorhanden!")
            Return False
        End If
        objSubEntryAttribute = objActiveSubEntry.Attributes(0)
        For Each objSubEntryAttribute In objActiveSubEntry.Attributes
            If objSubEntryAttribute.Name.ToUpper = SubEntryAttributeName.ToUpper Then
                objActiveSubentryAttribute = objSubEntryAttribute
                DocMode = XmlMode.xmlEntrySelected
                cMessage = ""
                Return True
            End If
        Next
        cMessage = ("Attribut nicht vorhanden!")
        Return False

    End Function


    '********************************************************************************************
    ' Setzen eines Untereintrags-Attributwertes (3. Ebene)
    ' -------------------------------
    ' Rckgabewerte: True = Attributwert wurde gesetzt
    '                false = Attributwert wurde nicht gesetzt
    ' Zeiger sitzt auf gefundenem Attribut
    ' Ein Untereintragsattribut muss vorher ausgewhlt werden
    '********************************************************************************************
    Public Function SetSubEntryAttribute(ByVal SubEntryAttributeValue As String) As Boolean
        'If Not DocMode = XmlMode.xmlEntrySelected Then
        '    cMessage = ("Kein Subentryattribut ausgewhlt!")
        '    Return False
        'End If

        Try
            If Not objActiveSubentryAttribute Is Nothing Then
                objActiveSubentryAttribute.Value = SubEntryAttributeValue
                cMessage = ""
                Return True
            Else
                cMessage = ("Es ist kein aktives Attribut vorhanden")
            End If
        Catch ex As Exception
            cMessage = ("Fehler bei bergabe an das Active Attribute")
        End Try
    End Function


    '********************************************************************************************
    ' Setzen eines Unter-Eintragswertes
    ' 
    ' Unter-Eintrag muss vorher gewhlt sein
    '********************************************************************************************
    Public Function SetSubEntryValue(ByVal NewSubEntryValue As String) As Boolean
        Try
            cMessage = ""
            objActiveSubEntry.InnerText = NewSubEntryValue
            Me.SaveDoc()
        Catch ex As Exception
            cMessage = "Fehler beim Schreiben (" & ex.Message.ToString & ")"
            Return False
        End Try
        cMessage = ""
        Return True
    End Function


    '********************************************************************************************
    ' Erzeugen eines arrays aus den Subentrienames
    ' -------------------------------
    ' Rckgabe: Array
    ' Zeiger muss vorher auf einem Eintrag stehen
    'Zeiger sitzt auf letztem Subentry
    '********************************************************************************************
    Public Function GetSubEntryArray() As Array
        If Not DocMode = XmlMode.xmlEntrySelected Then
            cMessage = ("Kein Subentryattribut ausgewhlt!")
            Exit Function
        End If
        Try
            Dim nNodeCount As Integer = objActiveEntry.ChildNodes.Count
            Dim tmpArray As Array = tmpArray.CreateInstance(GetType(String), nNodeCount)

            Dim objNodeSubEntry As XmlNode

            objNodeSubEntry = objActiveEntry.FirstChild
            Dim i As Integer = 0
            Do Until objNodeSubEntry Is Nothing
                objActiveSubEntry = objNodeSubEntry
                tmpArray.SetValue(objNodeSubEntry.Name, i)
                i += 1
                objNodeSubEntry = objNodeSubEntry.NextSibling
            Loop
            Return (tmpArray)
        Catch ex As Exception
            cMessage = ("Fehler bei bergabe an das Active Attribute")
        End Try

    End Function


    '********************************************************************************************
    ' Der Wert der Subentryeigenschaft wird bergeben
    ' -------------------------------
    ' Rckgabe: String
    ' Zeiger muss vorher auf einem Attribut stehen
    '********************************************************************************************
    Public Function GetSubentryAttribute() As String
        Try
            If Not objActiveSubentryAttribute Is Nothing Then
                cMessage = ""
                Return objActiveSubentryAttribute.Value
            End If
        Catch ex As Exception
            cMessage = ("Fehler bei Erhalt des Active Attribute")
        End Try
    End Function

    '********************************************************************************************
    ' Der Wert des Subentry wird bergeben
    ' -------------------------------
    ' Rckgabe: String
    ' Zeiger muss vorher auf einem aktiven Eintrag (sollte in dem Subentry sein) stehen
    '********************************************************************************************
    Public Function GetSubentryValue() As String
        Try
            If Not objActiveSubEntry Is Nothing Then
                cMessage = ""
                Return objActiveSubEntry.InnerText
            End If
        Catch ex As Exception
            cMessage = ("Fehler bei Erhalt des Aktiven Eintrags")
        End Try
    End Function

    '********************************************************************************************
    ' Der Wert des Subentry wird bergeben
    ' -------------------------------
    ' Rckgabe: String
    ' Zeiger muss vorher auf einem aktiven Eintrag (sollte in dem Subentry sein) stehen
    '********************************************************************************************
    Public Function GetSubentryName() As String
        Try
            If Not objActiveSubEntry Is Nothing Then
                cMessage = ""
                Return objActiveSubEntry.Name
            End If
        Catch ex As Exception
            cMessage = ("Fehler bei Erhalt des Aktiven Eintragsnamens")
        End Try
    End Function
End Class
